<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag restore backup copy
	 */
	 
	load_f('file_extension');
	
	class RESTOREBACKUP	{

		var $RECREATE_TABLES	=	FALSE;

		var $BACKUPDIR		=	FALSE;

		var $BASE		=	FALSE;

		var $TABLES		=	array();

		var $STRUCTURES		=	array();

		var $REPORT		=	'';

		function __construct()	  {

			global $__SQL, $__SESS, $__LANG;

			$this->SQL	=	&$__SQL;

			$this->SESS	=	&$__SESS;
			
			$this->LANG	=	&$__LANG;

			$this->FILEMARK	=	BASEDIR.'tmp/maintenance.conf';
			
			# Character set available as of MySQL 5.1.17
			
			$this->set_character_set('5.1.17');
		}

		public function directory($dir=FALSE)	{

			if (! $dir) return;
			
			$dir = safe_dir($dir);

			if (is_dir($dir)) $this->BACKUPDIR = $dir;
		}

		public function recreate_tables()	{

			$this->RECREATE_TABLES = TRUE;
		}

		public function base($BASE=FALSE)	{

			if ($BASE) $this->BASE = $BASE;
		}
		
		private function set_character_set($from_version)	{
		
			  $this->CHARACTER_SET = ' ';
			  
			  if (@version_compare($this->SQL->server_version(), $from_version, '>=')) $this->CHARACTER_SET = ' CHARACTER SET utf8 ';
		}

		private function loggeds_count()	{

			$COUNT = 0;

			$this->SQL->query("	SELECT `personID`
						FROM ebag.users
						WHERE `deleted` IS NOT NULL	");

			while ($RESULT = $this->SQL->RESULT->fetch_row())	{

				if ($this->SESS->is_logged($this->SQL->r_escape($RESULT[0]))) $COUNT++;
			}

			return $COUNT;
		}

		private function table_exists($TABLE)	{
		
			$TABLE = $this->SQL->escape($TABLE);

			$this->SQL->query("	SHOW TABLES 
						FROM $this->BASE LIKE '$TABLE' ");

			if ($this->SQL->num_rows()) return TRUE;

			return FALSE;
		}

		public function readdir()	{

			$dir = "$this->BACKUPDIR/$this->BASE";

			if ($dh = @opendir($dir))	{

				while (($file = readdir($dh)) != FALSE )	{

					if ((($file==".") or ($file=="..")) or (! is_file("$dir/$file"))) continue;
					
					$ext = file_extension($file);
					
					if ($ext == 'dat')	{
					
						$this->TABLES[] = str_replace('.dat','', $file);
					
					} elseif ($ext == 'str') {
					
						$this->STRUCTURES[] = str_replace('.str','', $file);
						   
					}	else 	  {
					
						continue;
					}
				}

				closedir($dh);
			}

			$T_COUNT = count($this->TABLES);
			$S_COUNT = count($this->STRUCTURES);

			if (	(! $T_COUNT) or
			   	(($T_COUNT != $S_COUNT) && ($this->RECREATE_TABLES))	) return FALSE;

			unset($this->STRUCTURES);

			return TRUE;
		}

		private function rep($line)	{

			$this->REPORT	.=	"$line\r\n";
		}

		public function report()	{

			return $this->REPORT;
		}

		public function restore()	{

			if ((! $this->BASE) or (! $this->BACKUPDIR)) return 'nobackupcopyfound';
			
			# verify the backup process is not in progress
			
			if (is_file("$this->BACKUPDIR/backup_started.tmp")) return 'backupinprogress';
			
			# also check for other process
			
			if (is_file($this->FILEMARK)) return 'maintenanceinprogress';
			
			# verify the connection
			
			if (! $this->SQL->connected()) return 'base_connection_error';

			# verify the base exists

			$this->SQL->query("	SELECT SCHEMA_NAME 
						FROM INFORMATION_SCHEMA.SCHEMATA 
						WHERE SCHEMA_NAME = '$this->BASE'	");

			if (! $this->SQL->num_rows()) return 'nodefinedbasefound';

			# verify the backup dir exists

			if (! $this->readdir()) return 'nobackupcopyfound';

			# check that is no more logges users

			if ($this->loggeds_count() != 1)	return 'moreofoneloggeds';

			# set time limit

			$time_limit	=	ini_get('max_execution_time');

			set_time_limit(0);

			# sets file mark for maintenance which will
			# redirect the users temporary from login

			@file_put_contents($this->FILEMARK, 'eBag_maintenance_in_progress');

			# recreate tables from scratch

			if ($this->RECREATE_TABLES)	{

				foreach ($this->TABLES as $TABLE)	{

					if ($CREATE_CMD	= @file_get_contents("$this->BACKUPDIR/$this->BASE/$TABLE.str"))	{

						# note: FALSE means dont stop if result is FALSE
	
						$this->SQL->query("	DROP TABLE 
									IF EXISTS $this->BASE.$TABLE		");

						$this->SQL->mysqli->select_db($this->BASE);

						$this->SQL->query($CREATE_CMD);
						
						$this->rep($this->LANG->get('recreatedtable', $TABLE));
					}
				}
			}

			# lock the tables

			$LOCK	=	'';

			$LOADS	=	array();

			foreach ($this->TABLES as $TABLE) 	{

				if (! $this->table_exists($TABLE)) continue; 

				if (! $LOCK) {$LOCK = "LOCK TABLES $this->BASE.$TABLE WRITE ";} else {$LOCK .= ", $this->BASE.$TABLE WRITE";}

				$LOADS[] = $TABLE;
			}

			unset($this->TABLES);

			if ((! count($LOADS)) or (! $LOCK)) 	{

				@unlink($this->FILEMARK);

				return 'nobackupcopyfound';
			}

			$this->SQL->transaction();

			$this->SQL->query($LOCK);

			# load the dumps into

			foreach ($LOADS as $TABLE)	{

				$FILE = "$this->BACKUPDIR/$this->BASE/$TABLE.dat";

				if ((is_file($FILE)) && (is_readable($FILE)))	{

					$this->SQL->query("	DELETE FROM $this->BASE.$TABLE ");

					$this->SQL->query("	LOAD DATA LOCAL INFILE '$FILE'
								REPLACE INTO TABLE $this->BASE.$TABLE
								$this->CHARACTER_SET
								FIELDS TERMINATED BY '\\t' ESCAPED BY '\\\'
								LINES TERMINATED BY '\\n'	");

					$this->rep($this->LANG->get('loadedtabledata', $TABLE));
				}
			}

			$this->SQL->commit();

			$this->SQL->query('UNLOCK TABLES');

			# delete file mark

			@unlink($this->FILEMARK);
			
			# restore time limit

			set_time_limit($time_limit);

			return TRUE;
		}
	
	}

?>
