	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */
	
	/**
	 * eBag calculator
	 */

	var LASTFUNC 	=	false;
	var BRACKET  	=	[];
	var MEMORY	=	[];
	var INC		=	0;
	MEMORY[INC] 	= 	'';
	var NEW_DISPLAY	=	true;
	var EQUAL	=	'';
	var LASTKEY	=	false;
	var DISPLAY	=	0;
	var NUM_SIGNS	=	[];
	var ERR_STR	=	'E';
	var RTL		=	0;
	var NO_TRANSL	=	false;

	/** keypress **/
	
	function fire(ID)	{

		document.getElementById(ID).focus();
		document.getElementById(ID).click(ID);
	}

	document.onkeydown = function(e)	{

		if (! e)	{
		
			if (! window.event) return;
			
			e	=	window.event;
		}

		e.cancelBubble = true;
		e.returnValue = false;

		if (e.stopPropagation) {
			e.stopPropagation();
			e.preventDefault();
		}
	}

	document.onkeypress = function(e)	{

		if (! e)	{
		
			if (! window.event) return;
			
			e	=	window.event;
		}
		
		e.preventDefault();
	}

	document.onkeyup = function(e)	{
			
		var CODE;

		if (! e)	{
		
			if (! window.event) return;
			
			e	=	window.event;
		}
		
		if (typeof(e.keyCode) == 'number')	{
		
			CODE	=	e.keyCode;

		}	else if (typeof(e.which) == 'number')	{
			
			CODE	=	e.which;

		}	else if (typeof(e.charCode) == 'number')	{
			
			CODE	=	e.charCode;

		}	else	{
		
			return;
		}

		if (((CODE == 98) || (CODE == 50)) && (LASTKEY == 88)) 	{

			fire('x2');
		}

		// numpad
		
		if ((CODE>95) && (CODE<106))	{

			fire('n_' + (CODE-96).toString());
		}
		
		// regular
		
		if ((CODE>47) && (CODE<58))	{

			fire('n_' + (CODE-48).toString());
		}

		switch (CODE)	{

			case(42)	:
			case(106)	:	fire('multiply');
						break;

			case(43)	:
			case(107)	:	fire('plus');
						break;

			case(45)	:
			case(109)	:	fire('minus');
						break;

			case(190)	:
			case(110)	:	fire('dot');
						break;

			case(47)	:
			case(111)	:	fire('divide');
						break;

			case(219)	:	fire('br_l');
						break;

			case(221)	:	fire('br_r');
						break;

			case(68)	:	fire('deg');
						break;

			case(73)	:	fire('inv');
						break;

			case(77)	:	fire('plusminus');
						break;

			case(65)	:	fire('abs');
						break;

			case(79)	:	if (LASTKEY == 76) fire('log');
						break;
						
			case(78)	:	if (LASTKEY == 76) {fire('ln');} else {fire('dot');}
						break;

			case(80)	:	fire('pi');
						break;

			case(83)	:	fire('sin');
						break;

			case(67)	:	fire('cos');
						break;

			case(84)	:	fire('tan');
						break;

			case(13)	:	fire('end');
						break;
						
			case(27)	:	fire('C_');							
						break;
		}

		LASTKEY = CODE;
		
		return false;
	}
	
	/** display & translate **/
	
	function load_numerics_set()	{
	  
		var i;
	 
		for (i=0; i<9; i++) {
		   
			NUM_SIGNS[i] = document.getElementById('n_' + i.toString()).value;
		}
		
		ERR_STR = document.getElementById('E_').value;
	}
	
	function display(str)	{
	  
		DISPLAY = str;
		
		document.getElementById('display').value  = translate(str.toString());
	}
	
	function translate(str)	{
	  
		if (NO_TRANSL) return str;
		
		if (str == 'E') return ERR_STR;
	  
		var i;
	  
		var splitted = str.split('');
		
		len = splitted.length;
		
		translated = '';
		
		for (i=0; i<len; i++) {
		  
			if (splitted[i] in NUM_SIGNS) { splitted[i] = NUM_SIGNS[splitted[i]]; }
			
			translated += splitted[i];
		}
		
		return translated;
	}

	/** calculations **/
	
	function is_deg()	{
	  
		return document.getElementById('deg').checked;
	}
	
	function is_inv()	{
	  
		return document.getElementById('inv').checked;
	}

	function _sin(val)	{

		if (is_inv())	{
	
			// inv sin	

			val = Math.asin(val) ;

			if (is_deg()) val = (180/Math.PI)*val;


		}	else	{

			// sin

			if (is_deg()) val = val*Math.PI/180;

			val = Math.sin(val);
		}

		return val;
	}

	function _cos(val)	{

		if (is_inv())	{
	
			// inv cos

			val = Math.acos(val) ;

			if (is_deg()) val = (180/Math.PI)*val;


		}	else	{

			// cos

			if (is_deg()) val = val*Math.PI/180;

			val = Math.cos(val);
		}

		return val;
	}

	function _tan(val)	{

		if (is_inv())	{
	
			// inv tan

			val = Math.tan(val) ;

			if (is_deg()) val = (180/Math.PI)*val;


		}	else	{

			// cos

			if (is_deg()) val = val*Math.PI/180;

			val = Math.atan(val);
		}

		return val;
	}

	function _x2(val)	{

		if (is_inv())	{

			val = Math.sqrt(val);

		}	else	  {

			val *= val;
		}
	
		return val;
	}

	function _log(val)	{

		if (is_inv())	{

			val = Math.pow(10,val);

		}	else	  {

			val = Math.log(val) / Math.log(10);
		}

		return val;
	}

	function _ln(val)	{

		if (is_inv())	{

			val = Math.exp(val);

		}	else	  {

			val = Math.log(val);
		}

		return val;
	}

	function _1perx(val)	{

		return 1/val;
	}

	function _abs(val)	{

		return Math.abs(val);
	}

	function show(str)	{

		var now;

		if (NEW_DISPLAY)	{

			NEW_DISPLAY	=	false;
			now		=	'';

		}	else	  {

			now = DISPLAY;
		}

		if (((now == '0') && (str!='.')) || (now == 'E') || (now=='')) now = '';

		if (str != 'E') str = now + str;

		display(str);
	}

	function newdisplay()	{

		NEW_DISPLAY	=	true;
	}

	function addval(str)	{

		LASTFUNC	=	false;		

		MEMORY[INC] += str;
	}

	function addfunc(str)	{

		if (LASTFUNC) {

			var N_MEMORY = '';

			if (len = MEMORY[INC].length) N_MEMORY += MEMORY[INC].substr(0, len-1);

			MEMORY[INC] = N_MEMORY;
		}

		LASTFUNC = true;

		MEMORY[INC] += str;
	}

	function number(str)	{

		if (EQUAL) EQUAL = '';

		show(str);
		addval(str);
	}

	function decimal()	{
	  
		if (DISPLAY.indexOf('.') != -1) return;

		if (EQUAL) EQUAL = '';

		show('.');
		addval('.');
	}

	function resetmem()	{

		INC		=	0;
		MEMORY		=	[];
		MEMORY[INC] 	=	'';
	}

	function reset_calc()	{

		resetmem();
		newdisplay();
		LASTFUNC	=	false;
		EQUAL		=	'';
		show('0');
	}
	
	function is_error(result)	{
	  
		if ((result == 'Infinity') || (result == '-Infinity') || (result == 'NaN') || (result == 'undefined') || (typeof result == 'undefined')) return true;
		
		return false;
	}

	function solve_part(part)	{

		result	=	false;

		// if is only one value return it

   		var REG = /^-{0,1}\d*\.{0,1}\d+$/;

    		if (REG.test(part)) return part;

		// or solve 

		try	{

			converted	=

			result = eval(part.replace('minus','-').replace('plus', '+'));

		}    catch(e)	{}

		if (is_error(result)) result = false;

		return result;
	}

	function solve()	{

		if ((EQUAL) && (! MEMORY[INC])) MEMORY[INC] = EQUAL;

		if ((! EQUAL) && (! MEMORY[INC])) MEMORY[INC] = '0';

		// first solve quantifiers		

		if (BRACKET[INC])	{

			result = solve_part(MEMORY[INC]);

			MEMORY[INC] = '';
			
			BRACKET[INC] = false;

			INC = INC-1;

			MEMORY[INC] += result;
		}

		// the solve the rest

		if (typeof MEMORY[INC] != 'undefined')	{

			result = solve_part(MEMORY[INC]);
		}

		if (result === false) {

			result		=	'E';
			resetmem();
			newdisplay();
			LASTFUNC	=	false;

		}	else	  {

			MEMORY[INC] = result;
		}

		display(result);

		LASTFUNC = false;

		return true;
	}

	function calculate(func)	{

		var memstr;
		var found;
		var parts;
		var len;
		var REG;
		var cheks;
		var last;
		var type = ['minus', 'plus', '*', '/'];

		switch (func)	{

			case('-')	:
			case('+')	:	if (! LASTFUNC)		{

							solve();
							newdisplay();	
							if (func=='-') {addfunc('minus');} else {addfunc('plus');}
						}

						break;

			case('/')	:
			case('*')	:	if (LASTFUNC) break;

						found = false;

						memstr = MEMORY[INC] + '';

						if (memstr.indexOf('minus') != -1) found = 'minus';
						if (memstr.indexOf('plus') != -1) found  =  'plus';

						if (! found) 	{

							solve();
							newdisplay();	
							addfunc(func);
							break;
						}

						elements = memstr.split(found);

						MEMORY[INC] = elements[0] + found;

						INC++;
						
						BRACKET[INC] = true;

						MEMORY[INC] = elements[1] + func;

						display(elements[1]);

						newdisplay();

						break;


			case('(')	:	INC++;
						MEMORY[INC] = '';
						break;
						
			case(')')	:	if (! INC) return;
			  
						solve();
						result = MEMORY[INC];
						MEMORY[INC] = '';
						INC = INC - 1;
						addval('(' + result + ')');
						break;


			case('+-')	:	if (! LASTFUNC)	{

							if (EQUAL) {addval(EQUAL); EQUAL='';}

							found = false;

							memstr = MEMORY[INC] + '';

							for (i=0; i<4; i++)	{

								if (memstr.indexOf(type[i]) != -1)	{

									parts	=	memstr.split(type[i]);

									len	=	parts.length;

									parts[len-1]	*=	(-1);

									MEMORY[INC]	=	parts.join(type[i]);

									found		=	true;
								}
							}

							if (! found)	MEMORY[INC] *= (-1);

							display((-1) * DISPLAY);
						}
						break;

			case('sin')	:
			case('cos')	:
			case('tan')	:	
			case('x2')	:
			case('1perx')	:
			case('abs')	:
			case('log')	:
			case('ln')	:	if (! LASTFUNC)	{

							if (EQUAL) {addval(EQUAL); EQUAL='';}

							found = false;

							memstr = MEMORY[INC] + '';

							for (i=0; i<4; i++)	{

								if (memstr.indexOf(type[i]) != -1)	{

									parts	=	memstr.split(type[i]);

									len	=	parts.length;

									result	=	eval('_' + func + '(\'' + parts[len-1] + '\')');

									parts[len-1]	=	result;

									MEMORY[INC]	=	parts.join(type[i]);

									found		=	true;
								}
							}

							if (! found)	{

								result	=	eval('_' + func + '(\'' + MEMORY[INC] + '\')');
								MEMORY[INC] = result;
							}

							if (is_error(result))	{

								result		=	'E';
								resetmem();
								newdisplay();
								LASTFUNC	=	false;

							}

							display(result);
						}
						
						break;
	
			case('pi')	:	REG = /^-{0,1}\d*\.{0,1}\d+$/;

    						if ((! MEMORY[INC]) || (REG.test(MEMORY[INC])))	{

							MEMORY[INC] = Math.PI + '';

						}	else	  {

							memstr = MEMORY[INC] + '';

							memstr = memstr.replace('minus','-').replace('plus', '+');

							last = false;

							// if is the last position

							checks = ['+', '-', '*', '/'];

							for (i=0; i<4; i++)	{

								if (memstr.charAt(memstr.length-1) == checks[i])	{

									last = true;

									addval(Math.PI);
								}
							}

							// if something is typed

							if (! last)	{

								for (i=0; i<4; i++)	{

									if (memstr.indexOf(type[i]) != -1)	{

										parts	=	memstr.split(type[i]);

										len	=	parts.length;

										parts[len-1]	=	Math.PI;

										MEMORY[INC]	=	parts.join(type[i]);
									}
								}
							}
						}

						LASTFUNC = false;

						display(Math.PI);

						break;

			case('end')	:	solve();
						if (MEMORY[INC]!='') EQUAL = MEMORY[INC];
						resetmem();
						newdisplay();
						LASTFUNC = false;

						break;
		}
	}
