<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * Congfiguration at install
	 */

	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('is_int_positive');
	load_f('is_real_positive');

	class Configuration	{
	
		var $data = array();
	
		public function set($section, $key, $value)	{

			 $this->data[$section][$key] = $value;
		}
		
		public function get($section, $key)	{
		
			 $value = FALSE;
		
			 if (isset($this->data[$section][$key])) $value = $this->data[$section][$key];
		      
			 return $value;
		}
		
		public function getall()	{
		
			return (array)$this->data;
		}
	}

	class InstallConfiguration	{
      
		var $CONF;
		
		var $LANG;
		
		var $report	=	array();
      
		function __construct($DATA)	{
		
			 global $__LANG;
			 
			 $this->LANG = &$__LANG;
		
			 $this->CONF = new Configuration();
		      
			 unset($DATA['install']);
		
			 $this->create_configuration_data($DATA);
		      
			 unset($DATA);
			 
			 $this->check_configuration_data();
			 
			 if (! $this->error())	{
			 
				# save configuration
			 
				$this->configuration_save();
				
				# create base
			 }
		}
	
		private function report($str)	{
		
			$this->report[] = $str;
		}
		
		public function getreport()	{
		
			return "\r\n\r\n".$this->LANG['errors_found']."\r\n\r\n".implode("\r\n", $this->report);
		}
		
		private function create_configuration_data($data)	{
		
			foreach ($data as $key=>$val)	{
			 
				list($key1, $key2) = mb_split('___', $key);
				  
				$this->CONF->set($key1, $key2, $val);
			}
		}
		
		private function report_translate()	{
		
			if (! count($this->report)) return;

			foreach ($this->report as $key=>$line)	{
			
			      if (isset($this->LANG[$line])) $this->report[$key] = '    -  '.$this->LANG[$line];
			}
		}
		
		public function error()	{
		
			if (count($this->report)) return TRUE;
			
			return FALSE;
		}
		
		private function is_writable_backup_dir($mysqli)	{
		
			$dir = safe_dir($this->CONF->get('backup', 'directory'));
		
			# www
	
			if (is_empty($dir) or (! is_dir($dir)) or (! is_writable($dir))) return FALSE;
			
			# mysql
	
			if ($mysqli)	{
			
				  $file = "$dir/is_writable.tmp";
				  
				  $file_esc = $mysqli->real_escape_string($file);
				  
				  @unlink($file);
				  
				  if (! $mysqli->query(" SELECT 'is_writable' INTO OUTFILE \"$file_esc\" ")) return FALSE;
				  
				  @unlink($file);
			}
			
			return TRUE;
		}
		
		private function configuration_save()	{
		
			$conf = ($this->CONF->getall());
			
			load_c('INICHANGE');
		
			$configuration	=	new INICHANGE();

			$configuration	->	open(BASEDIR.'conf/eBag.ini');

			foreach ($conf as $section=>$array) {
			
				foreach ($array as $key=>$val) $configuration->change($section, $key, $val); 
			}

			# save 

			$configuration	->	save(); 
		}
		
		private function check_configuration_data()	{
		
			$CONF = &$this->CONF;
		
			# check mysql connection

			$mysqli = @new mysqli($CONF->get('connection', 'host'), $CONF->get('connection', 'user'), $CONF->get('connection', 'password'));
				
			if ($mysqli->connect_error) { $this->report('cannot_connect_to_mysql'); $mysqli = FALSE; }
			
			# check user input
			
			if (is_empty($CONF->get('payments', 'currency'))) $this->report('undefined_currency');
			
			if (is_empty($CONF->get('payments', 'valuta'))) $this->report('undefined_valuta');
				
			$reals = array(	'exchange_rate'				=>	'payments',
					'default_tax_for_service'		=>	'tax',
					'default_tax_for_sales'			=>	'tax'	);
				
			foreach ($reals as $key=>$section)	{
			
				  if (! is_real_positive($CONF->get($section, $key))) $this->report('not_real_pos_'.$key);
			}
			
			$positives = array(	'default_deadline_for_service'		=>	'delivery',
						'default_deadline_for_shipment'		=>	'delivery',
						'dayscheck'				=>	'reminder',
						'increment'				=>	'display'	);
						
			foreach ($positives as $key=>$section)	{
			
				  if (! is_int_positive($CONF->get($section, $key))) $this->report('not_int_pos_'.$key);
			}
      
			unset($positives, $reals);
			
			# check smtp settings
			
			if (	(! is_empty($CONF->get('mail', 'host'))) or
				(! is_empty($CONF->get('mail', 'user'))) or
				(! is_empty($CONF->get('mail', 'port')))	)	{
				
					if ($CONF->get('mail', 'secure') == 'none') $CONF->set('mail', 'secure', '');
					
					$smtpcheckpassed = FALSE;
					
					if ($CONF->get('mail', 'password') == $CONF->get('mail', 're_password'))	{

						  load_c('SMTPCheck');
						  
						  $smtp = new SMTPCheck();
						  
						  $smtp->host($CONF->get('mail', 'host'));
						  $smtp->user($CONF->get('mail', 'user'));
						  $smtp->password($CONF->get('mail', 'password'));
						  $smtp->secure($CONF->get('mail', 'secure'));
						  $smtp->port($CONF->get('mail', 'port'));
						  
						  if ($smtp->check()) $smtpcheckpassed = TRUE;
						  
						  unset($smtp);
					}
					
					if (! $smtpcheckpassed) $this->report('smtp_settings_error');
				
			}	else	{
			
				$CONF->set('mail', 'host', '');
				$CONF->set('mail', 'user', '');
				$CONF->set('mail', 'password', '');
				$CONF->set('mail', 'port', '');
				$CONF->set('mail', 'secure', '');
			}
			
			$CONF->set('mail', 're_password', FALSE);
			
			# check backup directory
			
			if (($CONF->get('backup', 'directory') or $CONF->get('backup', 'daily_backup')) && (! $this->is_writable_backup_dir($mysqli))) $this->report('backup_directory_error');
				
			# check error report mail address 
			
			if (! is_empty($CONF->get('error', 'report_to')))	{

				if (function_exists('filter_var'))	{

					$valid=filter_var($CONF->get('error', 'report_to'), FILTER_VALIDATE_EMAIL);

				}	else	{

					$valid=@preg_match("/^[^@]*@[^@]*\.[^@]*$/", $CONF->get('error', 'report_to'));
				}

				if (! $valid) return $this->report('error_mail_syntax_error');
			}
			
			# check defined timezone & date
			
			if (! ini_set('date.timezone', $CONF->get('date', 'timezone'))) $this->report('timezone_error');
			
			load_f('date2sql');
			
			if (! date2sql(date($CONF->get('date', 'dateformat')), $CONF->get('date', 'dateformat'))) $this->report('date_format_error');
			
			# check for clamAV if is defined
			
			if ((! is_empty($CONF->get('virusscan', 'clamavpath'))) && (! is_readable($CONF->get('virusscan', 'clamavpath')))) $this->report('clamAV_path_error');
			
			# check secure port number
			
			$secure_port = (integer)$CONF->get('protocol', 'secure_port');
			
			if ($secure_port <= 0)	{$this->report('secure_port_error');} else {$CONF->set('protocol', 'secure_port', $secure_port);}
			
			$this->report_translate();
		}
	}
?>