<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 3.0.2
	 */

	/**
	 * eBag operating suplie request update
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('date2sql');

	load_f('datecompare');

	load_f('is_int_positive');

	load_f('is_real_positive');


	class SUPLIEREQUESTUPDATE	{

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->BASE		=	$_SESSION['BASE'];

			$this->STOCKBASE	=	'ebagstock';

			$this->PHPFORMAT	=	$__CONF->get('date', 'dateformat');

			$this->UNIT		=	new UNITloader($__CONF->get('cache', 'use_apc_cache'));
		}

		private function lock()	{

			$this->SQL->query("	LOCK TABLES	$this->BASE.suplie_request_header WRITE, 
								$this->BASE.suplie_request_data WRITE,
								$this->BASE.employees WRITE, 
								$this->STOCKBASE.operating_suplies WRITE	");
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		private function prepare($DATA)	{

			$CODES		=	$DATA['code'];
			$NAMES		=	$DATA['name'];
			$QUANTITYS	=	$DATA['quantity'];
			$UNITS		=	$DATA['unit'];

			$LENGTH	=	count($CODES);

			if (! $LENGTH) return 'usererror';

			$RETURN	=	array();

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$CODES[$i];
				$name		=	$NAMES[$i];
				$quantity	=	$QUANTITYS[$i];
				$unit		=	$UNITS[$i];

				# empty rows

				if ((is_empty($code)) && (is_empty($name)) && (is_empty($quantity)) && (is_empty($unit))) continue;

				# Initialize vars
				
				if (! isset($RETURN['code'])) 		$RETURN['code'] = array();
				if (! isset($RETURN['name'])) 		$RETURN['name'] = array();
				if (! isset($RETURN['quantity'])) 	$RETURN['quantity'] = array();
				if (! isset($RETURN['unit'])) 		$RETURN['unit'] = array();
				
				# user input

				if ((is_empty($code)) or (is_empty($name))) $RETURN = 'usererror';

				if ($this->UNIT->is_int_unit($unit))	{

					if (! is_int_positive($quantity)) $RETURN = 'usererror';
						
				}	else	 {

					if (! is_real_positive($quantity)) $RETURN = 'usererror';
				}				

				if ($RETURN == 'usererror') break;

				# service or artical exists

				$code	=	$RETURN['code'][]	=	$this->SQL->n_escape($code);

				$this->SQL->query("	SELECT `quantity` 
							FROM $this->STOCKBASE.operating_suplies 
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				if (! $this->SQL->num_rows())	{

					$RETURN = 'suplienotexist';

					break;
				}
				
				$RETURN['name'][]	=	$this->SQL->n_escape($name);
				$RETURN['quantity'][]	=	$quantity;
				$RETURN['unit'][]	=	$this->SQL->n_escape($unit);
			}
	
			if ((is_array($RETURN)) && (! count($RETURN))) $RETURN = 'usererror';

			return $RETURN;
		}

		public function update($ID, $DATA)	{

			$this->requestID	=	$this->SQL->escape($ID);

			$date_of_creation	=	date2sql($DATA['date_of_creation'], $this->PHPFORMAT);

			if (! $date_of_creation) return 'dateerror';

			if (datecompare($date_of_creation, '>', date("Y-m-d"))) return 'datebiggerthannow';

			$comment		=	$this->SQL->n_escape($DATA['comment']);

			if (! $DATA['employeeID']) return 'unexistentemployee';

			$employeeID		=	$this->SQL->escape($DATA['employeeID']);

			unset($DATA['date_of_creation'], $DATA['comment'], $DATA['employeeID']);

			$username = $this->SQL->n_escape($_SESSION['username']);

			# start transaction

			$this->SQL->transaction();
		
			$this->lock();

			# verify employee

			$this->SQL->query("	SELECT `employeename`
						FROM $this->BASE.employees
						WHERE `personID`='$employeeID' AND `deleted` IS NOT NULL	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'unexistentemployee';
			}

			# prepare service data
			
			$DATA = $this->prepare($DATA);

			if ((! isset($DATA['code'])) or (! $LENGTH = count($DATA['code']))) $DATA = 'usererror';

			if (! is_array($DATA)) 	{

				$this->SQL->rollback();

				$this->unlock();

				return $DATA;
			}

			# verify request existens

			$this->SQL->query("	SELECT `date_of_creation` 
						FROM $this->BASE.suplie_request_header
						WHERE `personID`='$this->requestID' AND `deleted` IS NOT NULL LIMIT 1	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'unexistentsuplierequest';
			}

			# update header

			$this->SQL->query("	UPDATE $this->BASE.suplie_request_header
						SET `date_of_creation`='$date_of_creation', `employeeID`='$employeeID', `user`=$username, `comment`=$comment
						WHERE `personID`='$this->requestID' AND `deleted` IS NOT NULL LIMIT 1	");

			# write back suplies into stock

			$this->SQL->query("	SELECT `code`, `quantity`
						FROM $this->BASE.suplie_request_data
						WHERE `requestID`='$this->requestID' AND `deleted` IS NOT NULL	");

			if ($this->SQL->num_rows())	{

				$this->SQL->IT1_RESULT = $this->SQL->RESULT;

				while ($IT1_RESULT = $this->SQL->IT1_RESULT->fetch_assoc())	{

					$code		=	$this->SQL->nr_escape($IT1_RESULT['code']);
					$quantity	=	$IT1_RESULT['quantity'];
	
					$this->SQL->query("	UPDATE $this->STOCKBASE.operating_suplies 
								SET quantity=quantity+$quantity
								WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");
				}
			}

			# delete suplies

			$this->SQL->query("	DELETE FROM $this->BASE.suplie_request_data
						WHERE `requestID`='$this->requestID'	");

			$COUNT		=	$LENGTH = count($DATA['code']);

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$DATA['code'][$i];
				$name		=	$DATA['name'][$i];
				$quantity	=	$DATA['quantity'][$i];
				$unit		=	$DATA['unit'][$i];

				# check for duplicates first

				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.suplie_request_data
							WHERE `code`=$code AND `requestID`='$this->requestID' AND `deleted` IS NOT NULL LIMIT 1	");

				if ($this->SQL->num_rows())	{

					# update
	
					$this->SQL->query("	UPDATE $this->BASE.suplie_request_data 
								SET quantity=quantity+$quantity
								WHERE `requestID`='$this->requestID' AND `deleted` IS NOT NULL LIMIT 1	");
				}	else	  {

					# insert

					$this->SQL->query("	INSERT INTO $this->BASE.suplie_request_data 
								VALUES  
								(NULL, '$this->requestID', $code, $name, '$quantity', $unit, '0')	");
				}

				# update stock quantity

				$this->SQL->query("	UPDATE $this->STOCKBASE.operating_suplies
							SET quantity=quantity-$quantity
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");
	
				# recheck stock that is not in minus

				$this->SQL->query("	SELECT `quantity` 
							FROM $this->STOCKBASE.operating_suplies
							WHERE quantity<0 AND `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				if ($this->SQL->num_rows())	{

					$errtxt	=	'notenoughsuplie->code->'.rtrim(ltrim($code, "'"), "'");

					break;
				}
			}

			if (isset($errtxt))	{

				$this->SQL->rollback();

				$this->unlock();

				return $errtxt;
			}

			$this->SQL->commit();

			$this->unlock();

			return TRUE;		
		}
	}

?>
