<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * save eBag item bonus data
	 */
	 
	# Turn off magic quotes runtime

	if (get_magic_quotes_runtime())	set_magic_quotes_runtime(false);
	
	if (get_magic_quotes_gpc())	{
	
		if (isset($_POST) && $_POST)	{
	
		      foreach ($_POST as $key=>$val) $_POST[$key] = stripslashes($val);
		}
	}
	
	# translate
	
	foreach(array('employee_id'=>'employeeID', 'stock_id'=>'stockID') as $key=>$val)	{
	
		if (isset($_POST[$key])) {$_POST[$val]=$_POST[$key]; unset($_POST[$key]);}
	}

	# read configuration

	include('../classes/INIload.class.php');

	$__CONF	= new INI_load('../conf/eBag.ini');
	
	# Sets session handler (important to refresh the session)

	include('../classes/Sessionhandler.class.php');

	$__SESS	=	new MysqlSession();
	$__SESS	->	set_session_handler();
	
	register_shutdown_function('session_write_close');

	session_start();
	
	if (isset($_SESSION['permission'])) {$_SESSION['permission'] = $_SESSION['permission'];} else {exit;}
	
	# not permitted to write (only to read)
	
	if (isset($_SESSION['year_is_closed'])) exit;
	
	# create connection
	
	$mysqli = new mysqli($__CONF->get('connection', 'host'), $__CONF->get('connection', 'user'), $__CONF->get('connection', 'password'));
	
	if ($mysqli->connect_error) exit;
	
	$mysqli->set_charset("utf8");

	$mysqli->query("SET NAMES 'utf8' COLLATE 'utf8_bin'");

	# define custom error handler

	function errorhandler($errno, $errstr)	{

		global $mysqli;
		
		$mysqli->query('UNLOCK TABLES');
		
		return FALSE;
	}

	set_error_handler('errorhandler');
	
	new Bonus;
	
	# exit codes 1=Set, 2=Unset, 3=Not found, 4=Invalid bonus set, 5=Type mismatch, 6=>Not product

	class Bonus	{
	  
		private $_data = array();
		
		private $_personID = FALSE;
		
		private $_bonus_value = NULL;
		  
		var $_bonuses =  array(	1=>'percent_production',
					2=>'fix_production',
					3=>'percent_services',
					4=>'fix_services',
					5=>'percent_product_sell',
					6=>'fix_product_sell',
					7=>'percent_service_sell',
					8=>'fix_service_sell'	);
					  
		var $_stock_tables = array(	1 => 'finished_goods_stock',
						2 => 'services'	);
					
		var $_stock_names = array(	1 => 'articalname',
						2 => 'servicename' );
		  
		function __construct()	{
		  
			global $mysqli, $__CONF;
			
			$this->SQL  = &$mysqli;
			
			# initalize bonus variables
			
			foreach ($this->_bonuses as $bonus) $this->_data[$bonus] = NULL;
			
			# read & verify $_POST

			foreach (array('employeeID', 'stock_type', 'stockID', 'code', 'bonus_type', 'bonus_value') as $varname)	{
			
				if (! isset($_POST[$varname])) {if ($varname == 'bonus_value') continue; exit('3');}
				
				$this->{'_'.$varname} = $this->SQL->real_escape_string(htmlspecialchars($_POST[$varname], $__CONF->get('specialchars', 'flag'), 'UTF-8', FALSE));
			}
			
			if (($this->_stock_type == 1) && (! in_array($this->_bonus_type, array(1,2,5,6)))) exit('5');
			
			if (($this->_stock_type == 2) && (! in_array($this->_bonus_type, array(3,4,7,8)))) exit('5');
			
			if (! $this->_bonus_value) $this->_bonus_value = NULL;
			
			if (($this->_bonus_value) && ((! is_numeric($this->_bonus_value)) or ($this->_bonus_value < 0))) exit('4');
			
			# Set base variables
		
			$this->_base = 'ebagdata'.$_SESSION['year'];
			
			$this->_stock_table = $this->_stock_tables[$this->_stock_type]; 
			$this->_stock_name  = $this->_stock_names[$this->_stock_type];
			
			# load existing data
			
			$this->load();
			
			$type = $this->_bonuses[$this->_bonus_type];
			
			if ($this->_data[$type] != $this->_bonus_value)	{
	
				$this->set($type, $this->_bonus_value);
				
				$this->save();
			}
			
			if ($this->_bonus_value) exit('1');
			
			exit('2');
		}
		  
		function _exit($exitcode)	{
		
			# rollback first
		
			$this->SQL->rollback();

			$this->SQL->query('UNLOCK TABLES');
			
			exit("$exitcode");
		}
		
		function dataNull()	{
		
			$is = TRUE;
			
			foreach ($this->_data as $val) {if ($val != NULL) {$is=FALSE; break;}}
		
			return $is;
		}
		
		function set($bonus, $val)	{
		
			$this->_data[$bonus] = $val;
			
			# check the opposite
			
			list($type, $based) = explode('_', $bonus, 2);
			
			if ($type == 'fix') {$type = 'percent';} else {$type = 'fix';}
			
			$this->_data[$type.'_'.$based] = NULL;
		}
		  
		function load()	{
		
			# start new transaction
		
			$this->SQL->query("LOCK TABLES	ebagstock.$this->_stock_table WRITE, 
							$this->_base.employees WRITE,
							$this->_base.employees_items_bonuses WRITE	");
							
			$this->SQL->autocommit(FALSE);

			# verify code (bonus to production can be assigned only to products)
			
			if ($this->_stock_table == 'services') {$ORIGIN = '';} else {$ORIGIN = ', `origin` ';}
			
			$result = $this->SQL->query("	SELECT  `personID` $ORIGIN FROM ebagstock.$this->_stock_table
							WHERE	`personID`='$this->_stockID'
							    AND `code`='$this->_code'
							    AND `deleted` IS NOT NULL  
							LIMIT 1	");
							
			if (! $result->num_rows) $this->_exit(3);
			
			if (($this->_bonus_type == 1) or ($this->_bonus_type == 2))	{
			
				$tmp = $result->fetch_assoc();
				
				if ($tmp['origin'] != 1) $this->_exit(6);
			}
			
			# verify employee
			
			$result = $this->SQL->query("	SELECT  `personID` FROM $this->_base.employees
							WHERE	`personID`='$this->_employeeID'
							    AND `deleted` IS NOT NULL  
							LIMIT 1	");
						
			if (! $result->num_rows) $this->_exit(3);
			
			# read bonus data
			
			$result = $this->SQL->query("	SELECT 	`personID`, `percent_production`, `fix_production`, 
								`percent_services`,`fix_services`, `percent_product_sell`, 
								`fix_product_sell`, `percent_service_sell`, `fix_service_sell`
							FROM $this->_base.employees_items_bonuses
							WHERE 	`employeeID`='$this->_employeeID' 
							    AND `stock_type`='$this->_stock_type'
							    AND `code`='$this->_code' 
							    AND `stockID`='$this->_stockID'
							    AND `deleted` IS NOT NULL
							LIMIT 1		");
				
			if (! $result->num_rows) return;
			
			$this->_data = $result->fetch_assoc();
			
			$this->_personID = $this->_data['personID'];
	
			unset($this->_data['personID']);
		}
		
		function save()	{
		
			# archive previous record

			if ($this->_personID)	{
			
				# todo : if the record is not used then delete it permanently

				$this->SQL->query("	UPDATE $this->_base.employees_items_bonuses 
							SET `deleted`=NULL 
							WHERE `personID`='$this->_personID' LIMIT 1	");
			}
			
			# form variables to mysql
			
			$isNull = $this->dataNull();
			
			$fields = '';
			
			for ($i=1; $i<9; $i++) {
			
				if ($this->_data[$this->_bonuses[$i]] == NULL) {$val = 'NULL';} else {$val = "'{$this->_data[$this->_bonuses[$i]]}'";}
				
				if ($fields) $fields .= ', ';
				
				$fields .= $val;
			}
			
			# do not store empty rows
			
			if (! $isNull)	{

				# insert
				
				$this->SQL->query("   INSERT INTO $this->_base.employees_items_bonuses
						      VALUES (	NULL, '$this->_employeeID', '$this->_stock_type', '$this->_stockID',
								'$this->_code', $fields, {$_SESSION['userID']}, '0' )	");
			}
			
			# commit
		
			$this->SQL->commit();

			$this->SQL->query('UNLOCK TABLES');
		}
	}
			
?>